@@ -28,7 +28,7 @@ class AlgorithmsTestCase(UflTestCase):
         c = Coefficient(element)
         f = Coefficient(element)
 
-        n = triangle.n
+        n = FacetNormal(triangle)
 
         a = u*v*dx
         L = f*v*dx
@@ -63,32 +63,6 @@ class AlgorithmsTestCase(UflTestCase):
             print extract_arguments(self.forms[2])
             print
 
-    def tearDown(self):
-        super(AlgorithmsTestCase, self).tearDown()
-
-    def test_flatten(self):
-        element = FiniteElement("CG", "triangle", 1)
-        a = Coefficient(element)
-        b = Coefficient(element)
-        c = Coefficient(element)
-        d = Coefficient(element)
-
-        a  = (a+b)+(c+d)
-        fa = flatten(a)
-        assert isinstance(a,  Sum) and len(a.operands())  == 2
-        assert isinstance(fa, Sum) and len(fa.operands()) == 4
-        aa, ab = a.operands()
-        assert isinstance(aa, Sum) and len(aa.operands()) == 2
-        assert isinstance(ab, Sum) and len(ab.operands()) == 2
-
-        a  = (a*b)*(c*d)
-        fa = flatten(a)
-        assert isinstance(a,  Product) and len(a.operands())  == 2
-        assert isinstance(fa, Product) and len(fa.operands()) == 4
-        aa, ab = a.operands()
-        assert isinstance(aa, Product) and len(aa.operands()) == 2
-        assert isinstance(ab, Product) and len(ab.operands()) == 2
-
     def test_arguments(self):
         assert self.arguments == tuple(extract_arguments(self.forms[0]))
         assert tuple(self.arguments[:1]) == tuple(extract_arguments(self.forms[1]))
@@ -100,7 +74,10 @@ class AlgorithmsTestCase(UflTestCase):
         #print elements(self.forms[2])
         #print unique_elements(self.forms[2])
         #print unique_classes(self.forms[2])
-        d = extract_duplications(self.forms[2].integrals(Measure.CELL)[0]._integrand)
+        b = self.forms[2]
+        integrals = b.integrals_by_type(Measure.CELL)
+        integrand = integrals[0].integrand()
+        d = extract_duplications(integrand)
         #pprint(list(d))
 
         element1 = FiniteElement("CG", triangle, 1)
@@ -127,7 +104,7 @@ class AlgorithmsTestCase(UflTestCase):
         def post(o, stack):
             poststore.append((o, len(stack)))
 
-        for itg in a.integrals(Measure.CELL):
+        for itg in a.integrals_by_type(Measure.CELL):
             walk(itg.integrand(), pre, post)
 
         self.assertEqual(prestore, [(p, 0), (v, 1), (f, 1)]) # NB! Sensitive to ordering of expressions.
@@ -174,11 +151,11 @@ class AlgorithmsTestCase(UflTestCase):
 
         u = TrialFunction(V1)
         v = TestFunction(V2)
-        self.assertGreater(u.count(), v.count())
+        self.assertGreater(u.number(), v.number())
 
-        u2 = Argument(V1)
-        v2 = Argument(V2)
-        self.assertLess(u2.count(), v2.count())
+        u2 = Argument(V1, 2)
+        v2 = Argument(V2, 3)
+        self.assertLess(u2.number(), v2.number())
 
         a = u*v*dx
         a_arg_degrees = [arg.element().degree() for arg in extract_arguments(a)]
