@@ -60,14 +60,14 @@ class SimplificationTestCase(UflTestCase):
         self.assertEqual(a, b)
 
         # Test simplification of flattened self-multiplication (may occur in algorithms)
-        a = Product(f,f,f)
-        b = f**3
-        self.assertEqual(a, b)
+        #a = Product(f,f,f)
+        #b = f**3
+        #self.assertEqual(a, b)
 
         # Test simplification of flattened self-multiplication (may occur in algorithms)
-        a = Product(f,f,f,f)
-        b = f**4
-        self.assertEqual(a, b)
+        #a = Product(f,f,f,f)
+        #b = f**4
+        #self.assertEqual(a, b)
 
     def test_sums(self):
         element = FiniteElement("CG", triangle, 1)
@@ -80,12 +80,12 @@ class SimplificationTestCase(UflTestCase):
         self.assertEqual(a, b)
 
         # Test collapsing of flattened sum (may occur in algorithms)
-        a = Sum(f, f, f)
-        b = 3*f
-        self.assertEqual(a, b)
-        a = Sum(f, f, f, f)
-        b = 4*f
-        self.assertEqual(a, b)
+        #a = Sum(f, f, f)
+        #b = 3*f
+        #self.assertEqual(a, b)
+        #a = Sum(f, f, f, f)
+        #b = 4*f
+        #self.assertEqual(a, b)
 
         # Test reordering of operands
         a = f + g
