@@ -14,7 +14,7 @@ class TestStrOfLiterals(UflTestCase):
         self.assertEqual(str(as_ufl(3.14)), "3.14")
 
     def test_str_zero(self):
-        x = triangle.x
+        x = SpatialCoordinate(triangle)
         self.assertEqual(str(as_ufl(0)), "0")
         self.assertEqual(str(0*x), "(0<(2,), ()>)") # TODO: Not very nice...
         self.assertEqual(str(0*x*x[Index(42)]), "(0<(2,), (Index(42),)>)") # TODO: Not very nice...
@@ -26,32 +26,32 @@ class TestStrOfLiterals(UflTestCase):
 class TestStrOfGeometricQuantities(UflTestCase):
 
     def test_str_coordinate(self):
-        self.assertEqual(str(triangle.x), "x")
-        self.assertEqual(str(triangle.x[0]), "(x)[0]") # FIXME: Get rid of extra ()
+        self.assertEqual(str(SpatialCoordinate(triangle)), "x")
+        self.assertEqual(str(SpatialCoordinate(triangle)[0]), "(x)[0]") # FIXME: Get rid of extra ()
 
     def test_str_normal(self):
-        self.assertEqual(str(triangle.n), "n")
-        self.assertEqual(str(triangle.n[0]), "(n)[0]") # FIXME: Get rid of extra ()
+        self.assertEqual(str(FacetNormal(triangle)), "n")
+        self.assertEqual(str(FacetNormal(triangle)[0]), "(n)[0]") # FIXME: Get rid of extra ()
 
     def test_str_circumradius(self):
-        self.assertEqual(str(triangle.circumradius), "circumradius") # TODO: Use a shorter name?
+        self.assertEqual(str(Circumradius(triangle)), "circumradius")
 
-    def test_str_cellsurfacearea(self):
-        self.assertEqual(str(triangle.surface_area), "surfacearea") # TODO: Use a shorter name?
+    #def test_str_cellsurfacearea(self):
+    #    self.assertEqual(str(CellSurfaceArea(triangle)), "surfacearea")
 
     def test_str_facetarea(self):
-        self.assertEqual(str(triangle.facet_area), "facetarea") # TODO: Use a shorter name?
+        self.assertEqual(str(FacetArea(triangle)), "facetarea")
 
     def test_str_volume(self):
-        self.assertEqual(str(triangle.volume), "volume") # TODO: Use a shorter name?
+        self.assertEqual(str(CellVolume(triangle)), "volume")
 
 class TestStrOfArguments(UflTestCase):
 
     def test_str_scalar_argument(self):
         v = TestFunction(FiniteElement("CG", triangle, 1))
         u = TrialFunction(FiniteElement("CG", triangle, 1))
-        self.assertEqual(str(v), "v_{-2}") # FIXME
-        self.assertEqual(str(u), "v_{-1}") # FIXME
+        self.assertEqual(str(v), "v_0")
+        self.assertEqual(str(u), "v_1")
 
     #def test_str_vector_argument(self): # FIXME
 
@@ -67,17 +67,17 @@ class TestStrOfArguments(UflTestCase):
 class TestStrOfTensors(UflTestCase):
 
     def test_str_list_vector(self):
-        x, y, z = tetrahedron.x
+        x, y, z = SpatialCoordinate(tetrahedron)
         v = as_vector((x, y, z))
         self.assertEqual(str(v), "[%s, %s, %s]" % (x, y, z))
 
     def test_str_list_vector_with_zero(self):
-        x, y, z = tetrahedron.x
+        x, y, z = SpatialCoordinate(tetrahedron)
         v = as_vector((x, 0, 0))
         self.assertEqual(str(v), "[%s, 0, 0]" % (x,))
 
     def test_str_list_matrix(self):
-        x, y = triangle.x
+        x, y = SpatialCoordinate(triangle)
         v = as_matrix(((2*x, 3*y),
                        (4*x, 5*y)))
         a = str(2*x)
@@ -87,7 +87,7 @@ class TestStrOfTensors(UflTestCase):
         self.assertEqual(str(v), "[\n  [%s, %s],\n  [%s, %s]\n]" % (a, b, c, d))
 
     def test_str_list_matrix_with_zero(self):
-        x, y = triangle.x
+        x, y = SpatialCoordinate(triangle)
         v = as_matrix(((2*x, 3*y),
                        (0, 0)))
         a = str(2*x)
