@@ -40,21 +40,26 @@ A very brief overview of the language contents follows:
 
 * Domains::
 
-    Domain, Region
+    Domain, ProductDomain
 
 * Cells::
 
-    Cell,
+    Cell, ProductCell, OuterProductCell,
     interval, triangle, tetrahedron,
     quadrilateral, hexahedron,
     cell1D, cell2D, cell3D,
 
+* Sobolev spaces::
+
+    L2, H1, H2, HDiv, HCurl
+
 * Elements::
 
     FiniteElement,
     MixedElement, VectorElement, TensorElement
     EnrichedElement, RestrictedElement,
-    TensorProductElement
+    TensorProductElement, OuterProductElement,
+    OuterProductVectorElement, HDiv, HCurl
 
 * Arguments::
 
@@ -74,11 +79,10 @@ A very brief overview of the language contents follows:
 
 * Geometric quantities::
 
-    SpatialCoordinate, FacetNormal,
-    CellVolume, Circumradius, CellSurfaceArea,
-    FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength, FacetDiameter,
-    LocalCoordinate, GeometryJacobi,
-    GeometryJacobiDeterminant, InverseGeometryJacobi
+    SpatialCoordinate,
+    FacetNormal, CellNormal,
+    CellVolume, Circumradius,
+    FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength,
 
 * Indices::
 
@@ -133,16 +137,17 @@ A very brief overview of the language contents follows:
 
 * Integral measures::
 
-    dx, ds, dS, dP, dE, dc
+    dx, ds, ds_b, ds_t, ds_tb, ds_v, dS, dS_h, dS_v, dP, dc, dE
 
 * Form transformations::
 
     rhs, lhs, system, functional,
-    replace, adjoint, action, energy_norm,
+    replace, replace_integral_domains,
+    adjoint, action, energy_norm,
     sensitivity_rhs, derivative
 """
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes and Anders Logg
+# Copyright (C) 2008-2014 Martin Sandve Alnes and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -162,10 +167,10 @@ A very brief overview of the language contents follows:
 # Modified by Kristian B. Oelgaard, 2009, 2011
 # Modified by Anders Logg, 2009.
 # Modified by Johannes Ring, 2014.
-#
-# Last changed: 2014-01-07
+# Modified by Andrew T. T. McRae, 2014
+# Modified by Lawrence Mitchell, 2014
 
-__version__ = "1.3.0"
+__version__ = "1.4.0"
 
 ########## README
 # Imports here should be what the user sees when doing "from ufl import *",
@@ -183,20 +188,23 @@ from ufl.log import get_handler, get_logger, set_handler, set_level, add_logfile
     UFLException, DEBUG, INFO, WARNING, ERROR, CRITICAL
 
 # Types for geometric quantities
-from ufl.geometry import (Cell, ProductCell,
-     SpatialCoordinate, FacetNormal,
-     CellVolume, Circumradius, CellSurfaceArea,
-     FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength, FacetDiameter,
-     LocalCoordinate, GeometryJacobi,
-     GeometryJacobiDeterminant, InverseGeometryJacobi)
+from ufl.cell import as_cell, Cell, ProductCell, OuterProductCell
+from ufl.domain import as_domain, Domain, ProductDomain
+from ufl.geometry import (
+    SpatialCoordinate,
+    FacetNormal, CellNormal,
+    CellVolume, Circumradius,
+    FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength,
+    )
 
-# Types for domain description
-from ufl.domains import Domain, Region
+# Sobolev spaces
+from ufl.sobolevspace import L2, H1, H2, HDiv, HCurl
 
 # Finite elements classes
 from ufl.finiteelement import FiniteElementBase, FiniteElement, \
     MixedElement, VectorElement, TensorElement, EnrichedElement, \
-    RestrictedElement, TensorProductElement
+    RestrictedElement, TensorProductElement, OuterProductElement, \
+    OuterProductVectorElement, HDiv, HCurl
 
 # Hook to extend predefined element families
 from ufl.finiteelement.elementlist import register_element, show_elements #, ufl_elements
@@ -244,11 +252,18 @@ from ufl.operators import rank, shape, \
                        jump, avg, cell_avg, facet_avg, \
                        elem_mult, elem_div, elem_pow, elem_op
 
+# Measure classes
+from ufl.measure import Measure, register_integral_type, integral_types
+
 # Form class
-from ufl.form import Form
+from ufl.form import Form, replace_integral_domains
 
 # Integral classes
-from ufl.integral import Integral, Measure, register_domain_type, ProductMeasure
+from ufl.integral import Integral
+
+# Special functions for Measure class
+# (ensure this is imported, since it attaches operators to Measure)
+import ufl.measureoperators as __measureoperators
 
 # Representations of transformed forms
 from ufl.formoperators import replace, derivative, action, energy_norm, rhs, lhs,\
@@ -259,7 +274,7 @@ from ufl.objects import \
     vertex, interval, triangle, tetrahedron, \
     quadrilateral, hexahedron, facet, cell1D, cell2D, cell3D, \
     i, j, k, l, p, q, r, s, \
-    dx, ds, dS, dP, dE, dc
+    dx, ds, ds_b, ds_t, ds_tb, ds_v, dS, dS_h, dS_v, dP, dc, dE
 
 # Useful constants
 from math import e, pi
@@ -268,16 +283,17 @@ __all__ = [
     'product',
     'get_handler', 'get_logger', 'set_handler', 'set_level', 'add_logfile',
     'UFLException', 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL',
-    'Cell', 'ProductCell',
-    'SpatialCoordinate', 'FacetNormal',
-    'CellVolume', 'Circumradius', 'CellSurfaceArea',
-    'FacetArea', 'MinFacetEdgeLength', 'MaxFacetEdgeLength', 'FacetDiameter',
-    'LocalCoordinate', 'GeometryJacobi',
-    'GeometryJacobiDeterminant', 'InverseGeometryJacobi',
-    'Domain', 'Region',
+    'as_cell', 'Cell', 'ProductCell', 'OuterProductCell',
+    'as_domain', 'Domain', 'ProductDomain',
+    'L2', 'H1', 'H2', 'HCurl', 'HDiv',
+    'SpatialCoordinate',
+    'CellVolume', 'Circumradius',
+    'FacetArea', 'MinFacetEdgeLength', 'MaxFacetEdgeLength',
+    'FacetNormal', 'CellNormal',
     'FiniteElementBase', 'FiniteElement',
     'MixedElement', 'VectorElement', 'TensorElement', 'EnrichedElement',
-    'RestrictedElement', 'TensorProductElement',
+    'RestrictedElement', 'TensorProductElement', 'OuterProductElement',
+    'OuterProductVectorElement', 'HDiv', 'HCurl',
     'register_element', 'show_elements',
     'Argument', 'TestFunction', 'TrialFunction',
     'Arguments', 'TestFunctions', 'TrialFunctions',
@@ -304,10 +320,10 @@ __all__ = [
     'jump', 'avg', 'cell_avg', 'facet_avg',
     'elem_mult', 'elem_div', 'elem_pow', 'elem_op',
     'Form',
-    'Integral', 'Measure', 'register_domain_type', 'ProductMeasure',
-    'replace', 'derivative', 'action', 'energy_norm', 'rhs', 'lhs',
+    'Integral', 'Measure', 'register_integral_type', 'integral_types',
+    'replace', 'replace_integral_domains', 'derivative', 'action', 'energy_norm', 'rhs', 'lhs',
     'system', 'functional', 'adjoint', 'sensitivity_rhs',
-    'dx', 'ds', 'dS', 'dP', 'dE', 'dc',
+    'dx', 'ds', 'ds_b', 'ds_t', 'ds_tb', 'ds_v', 'dS', 'dS_h', 'dS_v', 'dP', 'dc', 'dE',
     'vertex', 'interval', 'triangle', 'tetrahedron',
     'quadrilateral', 'hexahedron', 'facet',
     'cell1D', 'cell2D', 'cell3D',
