@@ -1,6 +1,6 @@
 "This module collects algorithms and utility functions operating on UFL objects."
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -18,32 +18,33 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
-#
-# First added:  2008-08-14
-# Last changed: 2011-10-11
 
-# Function for preprocessing a form
-from ufl.algorithms.preprocess import preprocess, preprocess_expression, extract_common_cell
+# Utilities for traversing over expression trees in different ways
+from ufl.algorithms.traversal import iter_expressions, \
+                                     traverse_terminals, traverse_unique_terminals, \
+                                     post_traversal, pre_traversal, \
+                                     post_walk, pre_walk, walk
 
 # Class for simple extraction of form meta data
 from ufl.algorithms.formdata import FormData
 
-# Utilities for traversing over expression trees in different ways
-from ufl.algorithms.traversal import iter_expressions, traverse_terminals, \
-                                     post_traversal, pre_traversal, \
-                                     post_walk, pre_walk, walk
+# Function for preprocessing a form
+from ufl.algorithms.preprocess import preprocess, preprocess_expression
 
 # Utilities for extracting information from forms and expressions
 from ufl.algorithms.analysis import extract_classes, extract_type, has_type, \
-                                    extract_arguments, extract_coefficients, extract_arguments_and_coefficients, \
+                                    extract_arguments, extract_coefficients, \
+                                    extract_arguments_and_coefficients, \
                                     extract_elements, extract_unique_elements, \
                                     extract_sub_elements, extract_unique_sub_elements, \
                                     extract_variables, extract_duplications, \
-                                    extract_max_quadrature_element_degree, estimate_quadrature_degree, \
+                                    extract_max_quadrature_element_degree, \
+                                    estimate_quadrature_degree, \
                                     sort_elements
 
 # Utilities for checking properties of forms
 from ufl.algorithms.predicates import is_multilinear
+from ufl.algorithms.signature import compute_form_signature
 
 # Utilities for error checking of forms
 from ufl.algorithms.checks import validate_form
@@ -56,14 +57,12 @@ from ufl.algorithms.transformer import Transformer, is_post_handler, \
                                        CopyTransformer, ufl2uflcopy, \
                                        VariableStripper, strip_variables
 from ufl.algorithms.replace import Replacer, replace
+from ufl.algorithms.change_to_reference import change_to_reference_grad
 from ufl.algorithms.expand_compounds import CompoundExpander, expand_compounds, \
                                             CompoundExpanderPreDiff, expand_compounds_prediff, \
                                             CompoundExpanderPostDiff, expand_compounds_postdiff
 from ufl.algorithms.estimate_degrees import SumDegreeEstimator, estimate_total_polynomial_degree
 from ufl.algorithms.argument_dependencies import ArgumentDependencyExtracter, extract_argument_dependencies, NotMultiLinearException
-from ufl.algorithms.deprecated import TreeFlattener, flatten, \
-                                      DuplicationMarker, mark_duplications, \
-                                      DuplicationPurger, purge_duplications
 from ufl.algorithms.renumbering import renumber_indices
 from ufl.algorithms.expand_indices import expand_indices, purge_list_tensors
 from ufl.algorithms.propagate_restrictions import propagate_restrictions
@@ -79,9 +78,6 @@ from ufl.algorithms.ad import expand_derivatives #, compute_diff, propagate_spat
 # Utilities for working with linearized computational graphs
 from ufl.algorithms.graph import Graph, format_graph, rebuild_tree, partition # TODO: Add more imports here
 
-# Utilities for tuple notation
-from ufl.algorithms.tuplenotation import as_form
-
 # Utilities for UFL object printing
 from ufl.algorithms.printing import integral_info, form_info, tree_format
 from ufl.algorithms.ufl2latex import ufl2latex, ufl2tex, ufl2pdf, forms2latexdocument
