@@ -1,6 +1,6 @@
 """Front-end for AD routines."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -68,28 +68,21 @@ def expand_derivatives(form, dim=None,
     In the returned expression g which is mathematically
     equivalent to expr, there are no VariableDerivative
     or CoefficientDerivative objects left, and Grad
-    objects have been propagated to Terminal nodes."""
+    objects have been propagated to Terminal nodes.
 
-    # Find geometric dimension. This is messy because of PyDOLFIN integration issues.
-    cell = form.cell()
-    gdim = None if cell is None else cell.geometric_dimension()
-    if dim is None:
-        dim = gdim
-    if gdim is not None:
-        ufl_assert(dim == gdim,
-                   "Expecting dim to match the geometric dimension, "+\
-                   "got dim=%r and gdim=%r." % (dim, gdim))
+    Note: Passing dim is now unnecessary, and the argument is ignored.
+    """
 
     def _expand_derivatives(expression):
         #print '_expand_derivatives:', expression
         # Expand compound expressions or not, in the future this
         # should be removed from here and applied on the outside.
         if apply_expand_compounds_before:
-            expression = expand_compounds(expression, dim)
+            expression = expand_compounds(expression)
             #print 'after expand_compounds', expression
 
         # Apply recursive forward mode AD
-        expression = apply_nested_forward_ad(expression, dim)
+        expression = apply_nested_forward_ad(expression)
 
         # FIXME: Form compilers assume expand_compounds have been applied.
         #        This means quite a bit of work to handle all compounds
@@ -99,8 +92,8 @@ def expand_derivatives(form, dim=None,
         #        alternative form compilers can then disable expand_compounds alltogether.
         if apply_expand_compounds_after:
             # FIXME: Test expand_compounds_postdiff, it should make this algorithm viable for existing FFC code
-            #expression = expand_compounds(expression, dim)
-            expression = expand_compounds_postdiff(expression, dim)
+            #expression = expand_compounds(expression)
+            expression = expand_compounds_postdiff(expression)
         return expression
 
     # Apply chosen algorithm to all integrands
