@@ -1,6 +1,6 @@
 """Utility algorithms for inspection of and information extraction from UFL objects in various ways."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -19,9 +19,6 @@
 #
 # Modified by Anders Logg, 2009-2010.
 # Modified by Johan Hake, 2010.
-#
-# First added:  2008-03-14
-# Last changed: 2013-01-02
 
 from itertools import izip, chain
 from collections import namedtuple
@@ -38,32 +35,52 @@ from ufl.argument import Argument
 from ufl.coefficient import Coefficient
 from ufl.variable import Variable
 from ufl.indexing import Index, MultiIndex
-from ufl.domains import Region, Domain
+from ufl.geometry import Domain
 from ufl.integral import Measure, Integral
 from ufl.form import Form
 from ufl.algorithms.traversal import iter_expressions, post_traversal, post_walk, traverse_terminals
 
-# Domain types (should probably be listed somewhere else)
-_domain_types = Measure._domain_types_tuple
-
 #--- Utilities to extract information from an expression ---
 
+# TODO: Some of these can possibly be optimised by implementing inlined stack based traversal algorithms
+
 def extract_classes(a):
     """Build a set of all unique Expr subclasses used in a.
     The argument a can be a Form, Integral or Expr."""
-    return set(o._uflclass for e in iter_expressions(a) \
-                        for o in post_traversal(e))
+    return set(o._uflclass
+               for e in iter_expressions(a)
+               for o in post_traversal(e))
 
 def extract_type(a, ufl_type):
     """Build a set of all objects of class ufl_type found in a.
     The argument a can be a Form, Integral or Expr."""
     if issubclass(ufl_type, Terminal):
-        return set(o for e in iter_expressions(a) \
-                     for o in traverse_terminals(e) \
-                     if isinstance(o, ufl_type))
-    return set(o for e in iter_expressions(a) \
-                 for o in post_traversal(e) \
-                 if isinstance(o, ufl_type))
+        return set(o for e in iter_expressions(a)
+                   for o in traverse_terminals(e)
+                   if isinstance(o, ufl_type))
+    return set(o for e in iter_expressions(a)
+               for o in post_traversal(e)
+               if isinstance(o, ufl_type))
+
+def expr_has_terminal_types(expr, ufl_types):
+    input = [expr]
+    while input:
+        e = input.pop()
+        ops = e.operands()
+        if ops:
+            input.extend(ops)
+        elif isinstance(e, ufl_types):
+            return True
+    return False
+
+def expr_has_types(expr, ufl_types):
+    input = [expr]
+    while input:
+        e = input.pop()
+        if isinstance(e, ufl_types):
+            return True
+        input.extend(e.operands())
+    return False
 
 def has_type(a, ufl_types):
     """Check if any class from ufl_types is found in a.
@@ -71,12 +88,11 @@ def has_type(a, ufl_types):
     if issubclass(ufl_types, Expr):
         ufl_types = (ufl_types,)
     if all(issubclass(ufl_type, Terminal) for ufl_type in ufl_types):
-        return any(isinstance(o, ufl_types) \
-                   for e in iter_expressions(a) \
-                   for o in traverse_terminals(e))
-    return any(isinstance(o, ufl_types) \
-               for e in iter_expressions(a) \
-               for o in post_traversal(e))
+        return any(expr_has_terminal_types(e, ufl_types)
+                   for e in iter_expressions(a))
+    else:
+        return any(expr_has_types(e, ufl_types)
+                   for e in iter_expressions(a))
 
 def extract_terminals(a):
     "Build a set of all Terminal objects in a."
@@ -84,10 +100,13 @@ def extract_terminals(a):
                  for o in post_traversal(e) \
                  if isinstance(o, Terminal))
 
+def sorted_by_number_and_part(seq):
+    return sorted(seq, key=lambda x: (x.number(), x.part()))
+
 def extract_arguments(a):
     """Build a sorted list of all arguments in a,
     which can be a Form, Integral or Expr."""
-    return sorted_by_count(extract_type(a, Argument))
+    return sorted_by_number_and_part(extract_type(a, Argument))
 
 def extract_coefficients(a):
     """Build a sorted list of all coefficients in a,
@@ -106,17 +125,17 @@ def extract_arguments_and_coefficients(a):
     arguments = [f for f in terminals if isinstance(f, Argument)]
     coefficients = [f for f in terminals if isinstance(f, Coefficient)]
 
-    # Build count: instance mappings, should be one to one
-    bfcounts = dict((f, f.count()) for f in arguments)
-    fcounts = dict((f, f.count()) for f in coefficients)
-
-    if len(bfcounts) != len(set(bfcounts.values())):
+    # Build number,part: instance mappings, should be one to one
+    bfnp = dict((f, (f.number(), f.part())) for f in arguments)
+    if len(bfnp) != len(set(bfnp.values())):
         msg = """\
-Found different Arguments with same counts.
+Found different Arguments with same number and part.
 Did you combine test or trial functions from different spaces?
 The Arguments found are:\n%s""" % "\n".join("  %s" % f for f in arguments)
         error(msg)
 
+    # Build count: instance mappings, should be one to one
+    fcounts = dict((f, f.count()) for f in coefficients)
     if len(fcounts) != len(set(fcounts.values())):
         msg = """\
 Found different coefficients with same counts.
@@ -124,13 +143,13 @@ The arguments found are:\n%s""" % "\n".join("  %s" % f for f in coefficients)
         error(msg)
 
     # Passed checks, so we can safely sort the instances by count
-    arguments = sorted_by_count(arguments)
+    arguments = sorted_by_number_and_part(arguments)
     coefficients = sorted_by_count(coefficients)
 
     return arguments, coefficients
 
-def build_argument_replace_map(arguments, coefficients, element_mapping=None):
-    """Create new Argument and Coefficient objects
+def build_coefficient_replace_map(coefficients, element_mapping=None):
+    """Create new Coefficient objects
     with count starting at 0. Return mapping from old
     to new objects, and lists of the new objects."""
     if element_mapping is None:
@@ -140,11 +159,9 @@ def build_argument_replace_map(arguments, coefficients, element_mapping=None):
             old_e = f.element()
             new_e = element_mapping.get(old_e, old_e)
             yield f.reconstruct(element=new_e, count=i)
-    new_arguments = list(remap(arguments))
     new_coefficients = list(remap(coefficients))
-    replace_map = dict(izip(chain(arguments, coefficients),
-                            chain(new_arguments, new_coefficients)))
-    return replace_map, new_arguments, new_coefficients
+    replace_map = dict(izip(coefficients, new_coefficients))
+    return new_coefficients, replace_map
 
 # alternative implementation, kept as an example:
 def _extract_coefficients(a):
@@ -190,7 +207,7 @@ def extract_element_map(elements):
 
 def extract_indices(expression):
     "Build a set of all Index objects used in expression."
-    info("Is this used for anything? Doesn't make much sense.")
+    warning("Is this used for anything? Doesn't make much sense.")
     multi_indices = extract_type(expression, MultiIndex)
     indices = set()
     for mi in multi_indices:
@@ -262,82 +279,35 @@ def estimate_quadrature_degree(integral):
     return sum(degrees)
 
 def unique_tuple(objects):
-    "Return sorted tuple of unique objects."
+    "Return tuple of unique objects."
     unique_objects = []
     for object in objects:
         if not object in unique_objects:
             unique_objects.append(object)
     return tuple(unique_objects)
 
-def extract_domain_data(form):
-    "Extract the domain_data attached to integrals of each domain type in form."
-    domain_data = {}
-    for integral in form.integrals():
-        domain_type = integral.domain_type()
-        data = integral.domain_data()
-        # Check that there is only one domain_data object for each integral type
-        existing_data = domain_data.get(domain_type)
-        if existing_data is None:
-            # Got no data before, store this one. May be None, that's fine.
-            domain_data[domain_type] = data
-        elif data is None:
-            # Already got data, getting no data is ok but don't overwrite.
-            pass
-        elif existing_data is not data:
-            # NB! Using 'is' because we're not assuming anything about domain_data type, not even an equals operator!
-            error("Found two domain data objects for same domain type '%s', only one is allowed." % str(domain_type))
-    return domain_data
-
 def extract_num_sub_domains(form):
-    "Extract the upper limit of sub domain ids for each domain type."
-    num_domains = {}
+    "Extract the upper limit of sub domain ids for each domain label/domain type."
+    num_sub_domains = {}
     for integral in form.integrals():
-        domain_type = integral.domain_type()
-        domain_id = integral.domain_id()
+        domain = integral.domain()
+        label = None if domain is None else domain.label()
+        if label not in num_sub_domains:
+            num_sub_domains[label] = {}
 
-        # TODO: This may need some redesign
-        max_domain_id = None
-        if isinstance(domain_id, int):
-            max_domain_id = domain_id
-        elif isinstance(domain_id, Region):
-            max_domain_id = max(domain_id.subdomain_ids())
-        elif isinstance(domain_id, Domain):
-            max_domain_id = None
+        subdomain_id = integral.subdomain_id()
+        max_subdomain_id = None
+        if isinstance(subdomain_id, int):
+            max_subdomain_id = subdomain_id
+        elif isinstance(subdomain_id, tuple):
+            max_subdomain_id = max(did for did in subdomain_id)
 
-        if max_domain_id is not None:
-            num_domains[domain_type] = max(num_domains.get(domain_type, 0), max_domain_id + 1)
-    return num_domains
+        integral_type = integral.integral_type()
+        if max_subdomain_id is not None:
+            prev = num_sub_domains[label].get(integral_type, 0)
+            num_sub_domains[label][integral_type] = max(prev, max_subdomain_id + 1)
 
-class IntegralData(object):
-    """Utility class with the members
-        (domain_type, domain_id, integrals, metadata)
-
-    where metadata is an empty dictionary that may be used for
-    associating metadata with each object.
-    """
-    __slots__ = ('domain_type', 'domain_id', 'integrals', 'metadata')
-    def __init__(self, domain_type, domain_id, integrals, metadata):
-        self.domain_type = domain_type
-        self.domain_id = domain_id
-        self.integrals = integrals
-        self.metadata = metadata
-
-    def __lt__(self, other):
-        # To preserve behaviour of extract_integral_data:
-        return ((self.domain_type, self.domain_id, self.integrals, self.metadata)
-                < (other.domain_type, other.domain_id, other.integrals, other.metadata))
-
-    def __eq__(self, other):
-        # Currently only used for tests:
-        return (self.domain_type == other.domain_type and
-                self.domain_id == other.domain_id and
-                self.integrals == other.integrals and
-                self.metadata == other.metadata)
-
-    def __str__(self):
-        return "IntegralData object over domain (%s, %s), with integrals:\n%s\nand metadata:\n%s" % (
-            self.domain_type, self.domain_id,
-            '\n\n'.join(map(str,self.integrals)), self.metadata)
+    return num_sub_domains
 
 def sort_elements(elements):
     """
