@@ -1,6 +1,6 @@
 """Functions to check the validity of forms."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -19,9 +19,6 @@
 #
 # Modified by Anders Logg, 2008-2009.
 # Modified by Mehdi Nikbakht, 2010.
-#
-# First added:  2008-03-14
-# Last changed: 2012-04-12
 
 from ufl.log import warning, error
 
@@ -47,6 +44,8 @@ def validate_form(form): # TODO: Can we make this return a list of errors instea
         #errors.append(msg)
         #return errors
 
+    # FIXME: There's a bunch of other checks we should do here.
+
     # FIXME: Add back check for multilinearity
     # Check that form is multilinear
     #if not is_multilinear(form):
@@ -59,14 +58,8 @@ def validate_form(form): # TODO: Can we make this return a list of errors instea
     if not domains:
         errors.append("Missing domain definition in form.")
 
-    top_domains = set(dom.top_domain() for dom in domains if dom is not None)
-    if not top_domains:
-        errors.append("Missing domain definition in form.")
-    elif len(top_domains) > 1:
-        warnings.append("Multiple top domain definitions in form: %s" % str(top_domains))
-
     # Check that cell is the same everywhere
-    cells = set(dom.cell() for dom in top_domains) - set((None,))
+    cells = set(dom.cell() for dom in domains) - set((None,))
     if not cells:
         errors.append("Missing cell definition in form.")
     elif len(cells) > 1:
@@ -89,17 +82,18 @@ def validate_form(form): # TODO: Can we make this return a list of errors instea
                     coefficients[c] = f
 
             elif isinstance(f, Argument):
-                c = f.count()
-                if c in arguments:
-                    g = arguments[c]
+                n = f.number()
+                p = f.part()
+                if (n,p) in arguments:
+                    g = arguments[(n,p)]
                     if not f is g:
-                        if c == -2: msg = "TestFunctions"
-                        elif c == -1: msg = "TrialFunctions"
-                        else: msg = "Arguments with same count"
+                        if n == 0: msg = "TestFunctions"
+                        elif n == 1: msg = "TrialFunctions"
+                        else: msg = "Arguments with same number and part"
                         msg = "Found different %s: %s and %s." % (msg, repr(f), repr(g))
                         errors.append(msg)
                 else:
-                    arguments[c] = f
+                    arguments[(n,p)] = f
 
     # Check that all integrands are scalar
     for expression in iter_expressions(form):
@@ -109,8 +103,8 @@ def validate_form(form): # TODO: Can we make this return a list of errors instea
 
     # Check that restrictions are permissible
     for integral in form.integrals():
-        # Only allow restricitions on interior facet integrals and surface measures
-        if integral.measure().domain_type() in (Measure.INTERIOR_FACET, Measure.SURFACE):
+        # Only allow restrictions on interior facet integrals and surface measures
+        if integral.integral_type() in (Measure.INTERIOR_FACET, Measure.INTERIOR_FACET_HORIZ, Measure.INTERIOR_FACET_VERT, Measure.SURFACE):
             check_restrictions(integral.integrand(), True)
         else:
             check_restrictions(integral.integrand(), False)
