@@ -1,6 +1,6 @@
 """Algorithms for estimating polynomial degrees of expressions."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes and Anders Logg
+# Copyright (C) 2008-2014 Martin Sandve Alnes and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -39,37 +39,72 @@ class SumDegreeEstimator(Transformer):
         self.element_replace_map = element_replace_map
 
     def constant_value(self, v):
-        "Constant values are constant. Duh."
+        "Constant values are constant."
         return 0
 
     def geometric_quantity(self, v):
-        "Most geometric quantities are cellwise constant."
-        ufl_assert(v.is_cellwise_constant(),
-                   "Missing handler for non-constant geometry type %s." % v._uflclass.__name__)
-        return 0
+        "Some geometric quantities are cellwise constant. Others are nonpolynomial and thus hard to estimate."
+        if v.is_cellwise_constant():
+            return 0
+        else:
+            # As a heuristic, just returning domain degree to bump up degree somewhat
+            x = v.domain().coordinates()
+            if x is None:
+                return 1
+            else:
+                return x.element().degree()
 
     def spatial_coordinate(self, v):
+        "A coordinate provides additional degrees depending on coordinate field of domain."
+        x = v.domain().coordinates()
+        if x is None:
+            return 1
+        else:
+            return x.element().degree()
+
+    def cell_coordinate(self, v):
         "A coordinate provides one additional degree."
         return 1
 
-    def local_coordinate(self, v):
-        "A coordinate provides one additional degree."
-        return 1
+    def argument(self, v):
+        """A form argument provides a degree depending on the element,
+        or the default degree if the element has no degree."""
+        return v.element().degree() # FIXME: Use component to improve accuracy for mixed elements
 
-    def form_argument(self, v):
+    def coefficient(self, v):
         """A form argument provides a degree depending on the element,
         or the default degree if the element has no degree."""
         e = v.element()
         e = self.element_replace_map.get(e,e)
-        d = e.degree() # FIXME: Use component to improve accuracy
-        return self.default_degree if d is None else d
+        d = e.degree() # FIXME: Use component to improve accuracy for mixed elements
+        if d is None:
+            d = self.default_degree
+        return d
 
     def _reduce_degree(self, v, f):
-        return max(f - 1, 0)
+        """Reduces the estimated degree by one; used when derivatives
+        are taken. Does not reduce the degree when OuterProduct elements
+        are involved."""
+        if isinstance(f, int):
+            return max(f-1, 0)
+        else:
+            # if tuple, do not reduce
+            return f
     def _add_degrees(self, v, *ops):
-        return sum(ops)
+        if all(isinstance(o, int) for o in ops):
+            return sum(ops)
+        else:
+            # we can add a slight hack here to handle things
+            # like adding 0 to (3, 3) [by expanding
+            # 0 to (0, 0) when making tempops]
+            tempops = [foo if isinstance(foo, tuple) else (foo, foo) for foo in ops]
+            return tuple(map(sum, zip(*tempops)))
     def _max_degrees(self, v, *ops):
-        return max(ops + (0,))
+        if all(isinstance(o, int) for o in ops):
+            return max(ops + (0,))
+        else:
+            tempops = [foo if isinstance(foo, tuple) else (foo, foo) for foo in ops]
+            return tuple(map(max, zip(*tempops)))
     def _not_handled(self, v, *args):
         error("Missing degree handler for type %s" % v._uflclass.__name__)
 
@@ -103,6 +138,7 @@ class SumDegreeEstimator(Transformer):
     # A sum takes the max degree of its operands:
     sum = _max_degrees
 
+    # TODO: Need a new algorithm which considers direction of derivatives of form arguments
     # A spatial derivative reduces the degree with one
     grad = _reduce_degree
     # Handling these types although they should not occur... please apply preprocessing before using this algorithm:
@@ -149,7 +185,7 @@ class SumDegreeEstimator(Transformer):
 
     def division(self, v, *ops):
         "Using the sum here is a heuristic. Consider e.g. (x+1)/(x-1)."
-        return sum(ops)
+        return self._add_degrees(v, *ops)
 
     def power(self, v, a, b):
         """If b is an integer:
@@ -159,11 +195,17 @@ class SumDegreeEstimator(Transformer):
         f, g = v.operands()
         try:
             gi = abs(int(g))
-            return a*gi
+            if isinstance(a, int):
+                return a*gi
+            else:
+                return tuple(foo*gi for foo in a)
         except:
             pass
         # Something to a non-integer power, this is just a heuristic with no background
-        return a*2
+        if isinstance(a, int):
+            return a*2
+        else:
+            return tuple(foo*2 for foo in a)
 
     def atan_2(self, v, a, b):
         """Using the heuristic
