@@ -2,7 +2,7 @@
 of UFL objects in the DOT graph visualization language,
 mostly intended for debugging purposers."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -18,9 +18,6 @@ mostly intended for debugging purposers."""
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
-#
-# First added:  2008-11-17
-# Last changed: 2011-10-14
 
 from itertools import chain
 
@@ -28,9 +25,9 @@ from ufl.log import error
 from ufl.expr import Expr
 from ufl.terminal import Terminal
 from ufl.form import Form
+from ufl.integral import Measure
 from ufl.variable import Variable
 from ufl.constantvalue import ScalarValue
-from ufl.geometry import FacetNormal
 from ufl.classes import FormArgument, MultiIndex, NegativeRestricted, PositiveRestricted
 from ufl.algorithms.multifunction import MultiFunction
 
@@ -60,22 +57,8 @@ class CompactLabeller(ReprLabeller):
     def form_argument(self, e):
         return self._function_mapping.get(id(e)) or str(e)
 
-    def spatial_coordinate(self, e):
-        return "x"
-    def facet_normal(self, e):
-        return "n"
-    def circumradius(self, e):
-        return "circumradius"
-    def cell_volume(self, e):
-        return "cell volume"
-    def cell_surface_area(self, e):
-        return "surface area"
-    def max_facet_edge_length(self, e):
-        return "max facet edge length"
-    def facet_area(self, e):
-        return "facet area"
-    def facet_diameter(self, e):
-        return "facet diameter"
+    def geometric_quantity(self, e):
+        return str(e)
 
     # Operators:
     def sum(self, e):
@@ -225,22 +208,27 @@ def ufl2dot(expression, formname="a", nodeoffset=0, begin=True, end=True, labeli
 
     if isinstance(expression, Form):
         form = expression
-        ci = form.integrals(Measure.CELL)
-        ei = form.integrals(Measure.EXTERIOR_FACET)
-        ii = form.integrals(Measure.INTERIOR_FACET)
-        pi = form.integrals(Measure.POINT)
-        mi = form.integrals(Measure.MACRO_CELL)
+        ci = form.integrals_by_type(Measure.CELL)
+        ei = form.integrals_by_type(Measure.EXTERIOR_FACET)
+        ebi = form.integrals_by_type(Measure.EXTERIOR_FACET_BOTTOM)
+        eti = form.integrals_by_type(Measure.EXTERIOR_FACET_TOP)
+        evi = form.integrals_by_type(Measure.EXTERIOR_FACET_VERT)
+        ii = form.integrals_by_type(Measure.INTERIOR_FACET)
+        ihi = form.integrals_by_type(Measure.INTERIOR_FACET_HORIZ)
+        ivi = form.integrals_by_type(Measure.INTERIOR_FACET_VERT)
+        pi = form.integrals_by_type(Measure.POINT)
+        mi = form.integrals_by_type(Measure.MACRO_CELL)
 
         subgraphs = []
         k = 0
-        for itgs in (ci, ei, ii, pi, mi):
+        for itgs in (ci, ei, ebi, eti, evi, ii, ihi, ivi, pi, mi):
             for itg in itgs:
                 prefix = "itg%d_" % k
-                integralkey = "%s%s" % (itg.measure().domain_type(), itg.measure().domain_id())
+                integralkey = "%s%s" % (itg.integral_type(), itg.subdomain_id())
 
-                integrallabel = "%s %s" % (itg.measure().domain_type().capitalize().replace("_", " "), "integral")
+                integrallabel = "%s %s" % (itg.integral_type().capitalize().replace("_", " "), "integral")
                 if len(itgs) > 1:
-                    integrallabel += " %s" % (itg.measure().domain_id(),)
+                    integrallabel += " %s" % (itg.subdomain_id(),)
 
                 integrand = itg.integrand()
 
@@ -272,7 +260,7 @@ def ufl2dot(expression, formname="a", nodeoffset=0, begin=True, end=True, labeli
         nodes = {}
         edges = []
 
-        build_entities(integrand, nodes, edges, nodeoffset, labeller)
+        build_entities(expression, nodes, edges, nodeoffset, '', labeller)
         entitylist = format_entities(nodes, edges)
         s = exprgraphformat % entitylist
 
