@@ -2,7 +2,7 @@
 either converting UFL expressions to new UFL expressions or
 converting UFL expressions to other representations."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -21,12 +21,10 @@ converting UFL expressions to other representations."""
 #
 # Modified by Anders Logg, 2008-2009.
 # Modified by Kristian B. Oelgaard, 2011
-#
-# First added:  2008-05-07
-# Last changed: 2011-10-21
 
 from itertools import chain
 
+import ufl
 from ufl.log import error, warning
 from ufl.assertions import ufl_assert
 from ufl.common import write_file, pdflatex, openpdf
@@ -47,7 +45,7 @@ from ufl.conditional import EQ, NE, LE, GE, LT, GT, Conditional
 from ufl.form import Form
 from ufl.integral import Measure
 from ufl.classes import terminal_classes
-from ufl.domains import Domain, Region
+from ufl.geometry import Domain
 
 # Other algorithms:
 from ufl.algorithms.preprocess import preprocess
@@ -112,8 +110,9 @@ def format_index(ii):
 def format_multi_index(ii, formatstring="%s"):
     return "".join(formatstring % format_index(i) for i in ii)
 
-def bfname(i):
-    return "{v_h^%d}" % i
+def bfname(i, p):
+    s = "" if p is None else (",%d"%(p,))
+    return "{v_h^{%d%s}}" % (i,s)
 
 def cfname(i):
     return "{w_h^%d}" % i
@@ -148,8 +147,8 @@ class Expression2LatexHandler(Transformer):
     def argument(self, o):
         # Using ^ for argument numbering and _ for indexing since indexing is more common than exponentiation
         if self.argument_names is None:
-            return bfname(o.count())
-        return self.argument_names[o.count()]
+            return bfname(o.number(), o.part())
+        return self.argument_names[(o.number(), o.part())]
 
     def coefficient(self, o):
         # Using ^ for coefficient numbering and _ for indexing since indexing is more common than exponentiation
@@ -377,23 +376,23 @@ def element2latex(element):
     e = str(element)
     e = e.replace("<", "")
     e = e.replace(">", "")
+    e = "fixme"
     return r"{\mbox{%s}}" % e
 
-domain_strings = { Measure.CELL: r"\Omega",
-                   Measure.EXTERIOR_FACET: r"\Gamma^{ext}",
-                   Measure.INTERIOR_FACET: r"\Gamma^{int}",
-                   Measure.POINT: r"\Gamma^{point}",
-                   Measure.MACRO_CELL: r"\Omega^{macro}",
-                   Measure.SURFACE: r"\Gamma^{surface}",
+domain_strings = { "cell": r"\Omega",
+                   "exterior_facet": r"\Gamma^{ext}",
+                   "exterior_facet_bottom": r"\Gamma_{bottom}^{ext}",
+                   "exterior_facet_top": r"\Gamma_{top}^{ext}",
+                   "exterior_facet_vert": r"\Gamma_{vert}^{ext}",
+                   "interior_facet": r"\Gamma^{int}",
+                   "interior_facet_horiz": r"\Gamma_{horiz}^{int}",
+                   "interior_facet_vert": r"\Gamma_{vert}^{int}",
+                   "point": r"\Gamma^{point}",
+                   "quadrature": r"\Gamma^{quadrature}",
+                   "macro_cell": r"\Omega^{macro}",
+                   "surface": r"\Gamma^{surface}",
                  }
 default_domain_string = "d(?)"
-dx_strings = { Measure.CELL: "dx",
-               Measure.EXTERIOR_FACET: "ds",
-               Measure.INTERIOR_FACET: "dS",
-               Measure.POINT: "dP",
-               Measure.MACRO_CELL: "dE",
-               Measure.SURFACE: "dc",
-             }
 
 def form2latex(form, formdata):
 
@@ -424,8 +423,10 @@ def form2latex(form, formdata):
 
     # Define arguments and coefficients
     lines = []
-    for i, f in enumerate(formdata.original_arguments):
-        lines.append("%s = %s \\in V_h^{%d} " % (argument_names[i], bfname(i), i))
+    for f in formdata.original_arguments:
+        i = f.number()
+        p = f.part()
+        lines.append("%s = %s \\in V_h^{%d} " % (argument_names[(i,p)], bfname(i,p), i)) # FIXME: Handle part in V_h
     for i, f in enumerate(formdata.original_coefficients):
         lines.append("%s = %s \\in W_h^{%d} " % (coefficient_names[i], cfname(i), i))
     if lines:
@@ -435,12 +436,17 @@ def form2latex(form, formdata):
 
     # Define variables
     handled_variables = set()
-    integrals = list(chain(form.integrals(Measure.CELL),
-                           form.integrals(Measure.EXTERIOR_FACET),
-                           form.integrals(Measure.INTERIOR_FACET),
-                           form.integrals(Measure.POINT),
-                           form.integrals(Measure.MACRO_CELL),
-                           form.integrals(Measure.SURFACE)))
+    integrals = list(chain(form.integrals_by_type(Measure.CELL),
+                           form.integrals_by_type(Measure.EXTERIOR_FACET),
+                           form.integrals_by_type(Measure.EXTERIOR_FACET_BOTTOM),
+                           form.integrals_by_type(Measure.EXTERIOR_FACET_TOP),
+                           form.integrals_by_type(Measure.EXTERIOR_FACET_VERT),
+                           form.integrals_by_type(Measure.INTERIOR_FACET),
+                           form.integrals_by_type(Measure.INTERIOR_FACET_HORIZ),
+                           form.integrals_by_type(Measure.INTERIOR_FACET_VERT),
+                           form.integrals_by_type(Measure.POINT),
+                           form.integrals_by_type(Measure.MACRO_CELL),
+                           form.integrals_by_type(Measure.SURFACE)))
     ufl_assert(len(integrals) == len(form.integrals()),
                "Not handling non-standard integral types!")
     lines = []
@@ -469,22 +475,25 @@ def form2latex(form, formdata):
         # TODO: Get list of expression strings instead of single expression!
         integrand_string = expression2latex(itg.integrand(), formdata.argument_names, formdata.coefficient_names)
 
-        domain_type = itg.domain_type()
-        dstr = domain_strings[domain_type]
+        integral_type = itg.integral_type()
+        dstr = domain_strings[integral_type]
 
-        domain_id = itg.domain_id()
-        if isinstance(domain_id, int):
-            dstr += "_{%d}" % domain_id
-        elif isinstance(domain_id, Domain) or domain_id == Measure.DOMAIN_ID_EVERYWHERE:
+        domain = itg.domain()
+        label = domain.label()
+        # TODO: Use domain label!
+
+        subdomain_id = itg.subdomain_id()
+        if isinstance(subdomain_id, int):
+            dstr += "_{%d}" % subdomain_id
+        elif subdomain_id == "everywhere":
             pass
-        elif domain_id == Measure.DOMAIN_ID_OTHERWISE:
+        elif subdomain_id == "otherwise":
             dstr += "_{\text{oth}}"
-        elif isinstance(domain_id, tuple):
-            dstr += "_{%s}" % domain_id
+        elif isinstance(subdomain_id, tuple):
+            dstr += "_{%s}" % subdomain_id
 
         b = p + "\\int_{%s}" % (dstr,)
-        dxstr = Measure._domain_types[domain_type]
-        dxstr = dx_strings[domain_type]
+        dxstr = ufl.measure.integral_type_to_measure_name[integral_type]
         c = "{ %s } \\,%s" % (integrand_string, dxstr)
         lines.append((a, b, c))
         a = "{}"
@@ -638,8 +647,8 @@ def form2code2latex(formdata):
     sections = [(title, body)]
 
     # Render each integral as a separate section
-    for itg in formdata.form.integrals(Measure.CELL):
-        title = "%s integral over domain %d" % (itg.domain_type(), itg.domain_id())
+    for itg in formdata.form.integrals_by_type(Measure.CELL):
+        title = "%s integral over domain %d" % (itg.integral_type(), itg.subdomain_id())
 
         G, partitions = integrand2code(itg.integrand(), formdata)
 
@@ -659,8 +668,7 @@ def forms2latexdocument(forms, uflfilename, compile=False):
     for form in forms:
 
         # Compute form data
-        form = preprocess(form)
-        form_data = form.form_data()
+        form_data = preprocess(form)
 
         # Generate LaTex code
         title = "Form %s" % form_data.name
