@@ -1,7 +1,7 @@
 """This module defines the class Argument and a number of related
 classes (functions), including TestFunction and TrialFunction."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -23,6 +23,7 @@ classes (functions), including TestFunction and TrialFunction."""
 # First added:  2008-03-14
 # Last changed: 2011-10-20
 
+from ufl.log import deprecate
 from ufl.assertions import ufl_assert
 from ufl.terminal import FormArgument
 from ufl.split_functions import split
@@ -32,56 +33,81 @@ from ufl.finiteelement import FiniteElementBase
 
 class Argument(FormArgument):
     """UFL value: Representation of an argument to a form."""
-    __slots__ = ("_count", "_repr", "_element",)
-    _globalcount = 0
+    __slots__ = ("_element", "_number", "_part", "_repr")
 
-    def __init__(self, element, count=None):
-        FormArgument.__init__(self, count, Argument)
+    def __init__(self, element, number, part=None):
+        FormArgument.__init__(self)
         ufl_assert(isinstance(element, FiniteElementBase),
-            "Expecting a FiniteElementBase instance.")
+                   "Expecting an element, not %s" % (element,))
+        ufl_assert(isinstance(number, int),
+                   "Expecting an int for number, not %s" % (number,))
+        ufl_assert(part is None or isinstance(part, int),
+                   "Expecting None or an int for part, not %s" % (part,))
         self._element = element
-        self._repr = "Argument(%r, %r)" % (self._element, self._count)
+        self._number = number
+        self._part = part
+        self._repr = "Argument(%r, %r, %r)" % (self._element, self._number, self._part)
 
-    def reconstruct(self, element=None, count=None):
-        if element is None or element == self._element:
+    def reconstruct(self, element=None, number=None, part=None):
+        if element is None or (element == self._element): # TODO: Is the == here a workaround for some bug?
             element = self._element
-        if count is None or count == self._count:
-            count = self._count
-        if count is self._count and element is self._element:
+        if number is None:
+            number = self._number
+        if part is None:
+            part = self._part
+        if number == self._number and part == self._part and element is self._element:
             return self
-        ufl_assert(isinstance(element, FiniteElementBase),
-                   "Expecting an element, not %s" % element)
-        ufl_assert(isinstance(count, int),
-                   "Expecting an int, not %s" % count)
         ufl_assert(element.value_shape() == self._element.value_shape(),
                    "Cannot reconstruct an Argument with a different value shape.")
-        return Argument(element, count)
+        return Argument(element, number, part)
 
     def element(self):
         return self._element
 
+    def number(self):
+        return self._number
+
+    def part(self):
+        return self._part
+
+    def count(self):
+        deprecate("The count of an Argument has been replaced with number() and part().")
+        ufl_assert(self.part() is None, "Deprecation transition for count() will not work with parts.")
+        return self.number() # I think this will work ok in most cases during the deprecation transition
+
     def shape(self):
         return self._element.value_shape()
 
-    def cell(self):
-        return self._element.cell()
-
-    def domain(self):
-        return self._element.domain()
-
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
         # TODO: Should in principle do like with Coefficient,
         # but that may currently simplify away some arguments
-        # we want to keep, or?
+        # we want to keep, or? See issue#13.
+        # When we can annotate zero with arguments, we can change this.
         return False
 
+    def domains(self):
+        "Return tuple of domains related to this terminal object."
+        return self._element.domains()
+
+    def signature_data(self, domain_numbering):
+        "Signature data for form arguments depend on the global numbering of the form arguments and domains."
+        s = self._element.signature_data(domain_numbering=domain_numbering)
+        return ("Argument", self._number, self._part) + s
+
     def __str__(self):
-        count = str(self._count)
-        if len(count) == 1:
-            return "v_%s" % count
+        number = str(self._number)
+        if len(number) == 1:
+            s = "v_%s" % number
         else:
-            return "v_{%s}" % count
+            s = "v_{%s}" % number
+        if self._part is not None:
+            part = str(self._part)
+            if len(part) == 1:
+                s = "%s^%s" % (s, part)
+            else:
+                s = "%s^{%s}" % (s, part)
+        return s
 
     def __repr__(self):
         return self._repr
@@ -90,41 +116,43 @@ class Argument(FormArgument):
         """Deliberately comparing exact type and not using isinstance here,
         meaning eventual subclasses must reimplement this function to work
         correctly, and instances of this class will compare not equal to
-        instances of eventual subclasses. The overloading allows 
+        instances of eventual subclasses. The overloading allows
         subclasses to distinguish between test and trial functions
         with a different non-ufl payload, such as dolfin FunctionSpace
-        with different mesh. This is necessary because of the test/trial
-        function hack with -2/-1 counts which always gives
-        TestFunction(V) == TestFunction(V) from a pure ufl point of view.
+        with different mesh. This is necessary because arguments with the
+        same element and argument number are always equal from a pure ufl
+        point of view, e.g. TestFunction(V1) == TestFunction(V2) if V1 and V2
+        are the same ufl element but different dolfin function spaces.
         """
         return (type(self) == type(other) and
-                self._count == other._count and
+                self._number == other._number and
+                self._part == other._part and
                 self._element == other._element)
 
 
 # --- Helper functions for pretty syntax ---
 
-def TestFunction(element):
+def TestFunction(element, part=None):
     """UFL value: Create a test function argument to a form."""
-    return Argument(element, -2)
+    return Argument(element, 0, part)
 
-def TrialFunction(element):
+def TrialFunction(element, part=None):
     """UFL value: Create a trial function argument to a form."""
-    return Argument(element, -1)
+    return Argument(element, 1, part)
 
 # --- Helper functions for creating subfunctions on mixed elements ---
 
-def Arguments(element):
+def Arguments(element, number):
     """UFL value: Create an Argument in a mixed space, and return a
     tuple with the function components corresponding to the subelements."""
-    return split(Argument(element))
+    return split(Argument(element, number))
 
 def TestFunctions(element):
     """UFL value: Create a TestFunction in a mixed space, and return a
     tuple with the function components corresponding to the subelements."""
-    return split(TestFunction(element))
+    return Arguments(element, 0)
 
 def TrialFunctions(element):
     """UFL value: Create a TrialFunction in a mixed space, and return a
     tuple with the function components corresponding to the subelements."""
-    return split(TrialFunction(element))
+    return Arguments(element, 1)
