@@ -4,7 +4,7 @@ implementation details not exposed through the default ufl namespace.
 It also contains functionality used by algorithms for dealing with groups
 of classes, and for mapping types to different handler functions."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -23,38 +23,44 @@ of classes, and for mapping types to different handler functions."""
 #
 # Modified by Anders Logg, 2009.
 # Modified by Kristian B. Oelgaard, 2011
-#
-# First added:  2008-08-15
-# Last changed: 2013-03-29
+# Modified by Andrew T. T. McRae, 2014
 
 from ufl.assertions import ufl_assert
 
 # Elements
 from ufl.finiteelement import (FiniteElementBase, FiniteElement,
     MixedElement, VectorElement, TensorElement,
-    EnrichedElement, RestrictedElement, TensorProductElement)
+    EnrichedElement, RestrictedElement, TensorProductElement,
+    OuterProductElement, OuterProductVectorElement)
 
 # Base class for all expressions
 from ufl.expr import Expr
 
 # Terminal types
-from ufl.terminal import Terminal, FormArgument, UtilityType, Data
-from ufl.constantvalue import ConstantValue, IndexAnnotated, Zero, ScalarValue,\
-    FloatValue, IntValue, Identity, PermutationSymbol
+from ufl.terminal import Terminal, FormArgument, UtilityType
+from ufl.constantvalue import (ConstantValue, IndexAnnotated,
+    Zero, ScalarValue, FloatValue, IntValue, Identity, PermutationSymbol)
 from ufl.argument import Argument, TestFunction, TrialFunction
 from ufl.coefficient import (Coefficient, ConstantBase,
     VectorConstant, TensorConstant, Constant)
-from ufl.geometry import (Cell, ProductCell,
-    GeometricQuantity,
-    SpatialCoordinate, FacetNormal,
-    CellVolume, Circumradius, CellSurfaceArea,
-    FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength, FacetDiameter,
-    LocalCoordinate, GeometryJacobi,
-    GeometryJacobiDeterminant, InverseGeometryJacobi)
+from ufl.cell import Cell, ProductCell, OuterProductCell
+from ufl.domain import Domain, ProductDomain
+from ufl.geometry import (
+    GeometricQuantity, GeometricCellQuantity, GeometricFacetQuantity,
+    SpatialCoordinate, CellCoordinate, FacetCoordinate,
+    CellOrigin, FacetOrigin, CellFacetOrigin,
+    Jacobian, JacobianDeterminant, JacobianInverse,
+    FacetJacobian, FacetJacobianDeterminant, FacetJacobianInverse,
+    CellFacetJacobian, CellFacetJacobianDeterminant, CellFacetJacobianInverse,
+    FacetNormal, CellNormal,
+    CellVolume, Circumradius,
+    FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength,
+    CellOrientation, QuadratureWeight,
+    )
 from ufl.indexing import IndexBase, FixedIndex, Index, MultiIndex
 
 # Operator types
-from ufl.operatorbase import Operator, WrapperType, AlgebraOperator, Tuple
+from ufl.operatorbase import Operator, WrapperType, AlgebraOperator
 from ufl.indexed import Indexed
 from ufl.indexsum import IndexSum
 from ufl.variable import Variable, Label
@@ -66,14 +72,16 @@ from ufl.mathfunctions import MathFunction, Sqrt, Exp, Ln, Erf,\
     Cos, Sin, Tan, Cosh, Sinh, Tanh, Acos, Asin, Atan, Atan2, \
     BesselFunction, BesselJ, BesselY, BesselI, BesselK
 from ufl.differentiation import Derivative, CompoundDerivative, CoefficientDerivative,\
-    VariableDerivative, Grad, Div, Curl, NablaGrad, NablaDiv
+    VariableDerivative, Grad, Div, Curl, NablaGrad, NablaDiv, ReferenceGrad
 from ufl.conditional import Condition, BinaryCondition,\
     EQ, NE, LE, GE, LT, GT,\
     AndCondition, OrCondition, NotCondition, Conditional
 from ufl.restriction import Restricted, PositiveRestricted, NegativeRestricted, CellAvg, FacetAvg
+from ufl.exprcontainers import ExprList, ExprMapping
 
 # Higher level abstractions
-from ufl.integral import Measure, ProductMeasure, Integral
+from ufl.measure import Measure, MeasureSum, MeasureProduct
+from ufl.integral import Integral
 from ufl.form import Form
 from ufl.equation import Equation
 
