@@ -1,6 +1,6 @@
 "This module defines classes representing constant values."
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -18,9 +18,6 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2011.
-#
-# First added:  2008-11-01
-# Last changed: 2011-11-10
 
 from itertools import izip
 from ufl.log import warning, error
@@ -30,23 +27,11 @@ from ufl.terminal import Terminal
 from ufl.indexing import Index, FixedIndex
 from ufl.common import EmptyDict
 
-#--- "Low level" scalar types ---
-
-# TODO: Using high precision float from numpy if available?
-int_type = int
-float_type = float
-python_scalar_types = (int, float)
-#try:
-#    import numpy
-#    int_type = numpy.int64
-#    float_type = numpy.float96
-#    python_scalar_types += (int_type, float_type)
-#except:
-#    pass
+#--- Helper functions imported here for compatibility---
+from ufl.checks import is_python_scalar, is_ufl_scalar, is_true_ufl_scalar
 
 # Precision for float formatting
 precision = None
-
 def format_float(x):
     "Format float value based on global UFL precision."
     if precision is None:
@@ -65,8 +50,9 @@ class ConstantValue(Terminal):
         "Return whether this expression is spatially constant over each cell."
         return True
 
-    def __getnewargs__(self):
-        return (self._dim,)
+    def domains(self):
+        "Return tuple of domains related to this terminal object."
+        return ()
 
 class IndexAnnotated(ConstantValue):
     """Class to annotate expressions that don't depend on
@@ -85,6 +71,42 @@ class IndexAnnotated(ConstantValue):
         if (set(self._free_indices) ^ set(self._index_dimensions.keys())):
             error("Index set mismatch.")
 
+#--- Class for representing abstract constant symbol only for use internally in form compilers
+#class AbstractSymbol(ConstantValue):
+#    "UFL literal type: Representation of a constant valued symbol with unknown properties."
+#    __slots__ = ("_name", "_shape")
+#    def __init__(self, name, shape):
+#        ConstantValue.__init__(self)
+#        self._name = name
+#        self._shape = shape
+#
+#    def __getnewargs__(self):
+#        return (self._name, self._shape)
+#
+#    def reconstruct(self, name=None):
+#        if name is None:
+#            name = self._name
+#        return AbstractSymbol(name, self._shape)
+#
+#    def shape(self):
+#        return self._shape
+#
+#    def free_indices(self):
+#        return ()
+#
+#    def index_dimensions(self):
+#        return EmptyDict
+#
+#    def __str__(self):
+#        return "<Abstract symbol named '%s' with shape %s>" % (self._name, self._shape)
+#
+#    def __repr__(self):
+#        return "AbstractSymbol(%r, %r)" % (self._name, self._shape)
+#
+#    def __eq__(self, other):
+#        return isinstance(other, AbstractSymbol) and self._name == other._name and self._shape == other._shape
+
+
 #--- Class for representing zero tensors of different shapes ---
 
 # TODO: Add geometric dimension and Argument dependencies to Zero?
@@ -219,7 +241,7 @@ class ScalarValue(IndexAnnotated):
         but ufl-python comparisons like
             IntValue(1) == 1.0
             FloatValue(1.0) == 1
-        can still succeed. These will however not have the same 
+        can still succeed. These will however not have the same
         hash value and therefore not collide in a dict."""
         if not isinstance(other, self._uflclass):
             return isinstance(other, (int,float)) and other == self._value
@@ -246,7 +268,7 @@ class FloatValue(ScalarValue):
     __slots__ = ()
     def __init__(self, value, shape=(), free_indices=(), index_dimensions=None):
         ScalarValue.__init__(self,
-                             float_type(value),
+                             float(value),
                              shape,
                              free_indices,
                              index_dimensions)
@@ -275,7 +297,7 @@ class IntValue(ScalarValue):
 
     def __init__(self, value, shape=(), free_indices=(), index_dimensions=None):
         if not hasattr(self, '_value'):
-            ScalarValue.__init__(self, int_type(value), shape, free_indices, index_dimensions)
+            ScalarValue.__init__(self, int(value), shape, free_indices, index_dimensions)
 
     def __repr__(self):
         return "%s(%s, %s, %s, %s)" % (type(self).__name__, repr(self._value),
@@ -314,6 +336,8 @@ class Identity(ConstantValue):
     def __eq__(self, other):
         return isinstance(other, Identity) and self._dim == other._dim
 
+    def __getnewargs__(self):
+        return (self._dim,)
 
 #--- Permutation symbol ---
 
@@ -365,30 +389,16 @@ class PermutationSymbol(ConstantValue):
                     return Zero()
         return result
 
-#--- Helper functions ---
-
-def is_python_scalar(expression):
-    "Return True iff expression is of a Python scalar type."
-    return isinstance(expression, python_scalar_types)
-
-def is_ufl_scalar(expression):
-    """Return True iff expression is scalar-valued,
-    but possibly containing free indices."""
-    return isinstance(expression, Expr) and not expression.shape()
-
-def is_true_ufl_scalar(expression):
-    """Return True iff expression is scalar-valued,
-    with no free indices."""
-    return isinstance(expression, Expr) and \
-        not (expression.shape() or expression.free_indices())
+    def __getnewargs__(self):
+        return (self._dim,)
 
 def as_ufl(expression):
     "Converts expression to an Expr if possible."
     if isinstance(expression, Expr):
         return expression
-    if isinstance(expression, (int, int_type)):
+    if isinstance(expression, int):
         return IntValue(expression)
-    if isinstance(expression, (float, float_type)):
+    if isinstance(expression, float):
         return FloatValue(expression)
     error(("Invalid type conversion: %s can not be converted to any UFL type.\n"+\
            "The representation of the object is:\n%r") % (type(expression), expression))
