@@ -1,6 +1,6 @@
 "Differential operators."
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -18,16 +18,14 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009.
-#
-# First added:  2008-03-14
-# Last changed: 2011-11-10
 
 from ufl.log import warning, error
 from ufl.assertions import ufl_assert
 from ufl.common import subdict, mergedicts, EmptyDict
 from ufl.expr import Expr
-from ufl.terminal import Terminal, Data
-from ufl.operatorbase import Operator, Tuple
+from ufl.terminal import Terminal
+from ufl.operatorbase import Operator
+from ufl.exprcontainers import ExprList, ExprMapping
 from ufl.constantvalue import Zero
 from ufl.indexing import Index, FixedIndex, MultiIndex, as_multi_index
 from ufl.indexed import Indexed
@@ -50,13 +48,12 @@ class CoefficientDerivative(Derivative):
                  "_coefficient_derivatives")
 
     def __new__(cls, integrand, coefficients, arguments, coefficient_derivatives):
-        ufl_assert(isinstance(coefficients, Tuple),
-            "Expecting Tuple instance with Coefficients.")
-        ufl_assert(isinstance(arguments, Tuple),
-            "Expecting Tuple instance with Arguments.")
-        ufl_assert(isinstance(coefficient_derivatives, (dict, Data)),
-                   "Expecting a dict for coefficient derivatives.")
-
+        ufl_assert(isinstance(coefficients, ExprList),
+                   "Expecting ExprList instance with Coefficients.")
+        ufl_assert(isinstance(arguments, ExprList),
+                   "Expecting ExprList instance with Arguments.")
+        ufl_assert(isinstance(coefficient_derivatives, ExprMapping),
+                   "Expecting ExprMapping for coefficient derivatives.")
         if isinstance(integrand, Zero):
             return integrand
         return Derivative.__new__(cls)
@@ -66,10 +63,10 @@ class CoefficientDerivative(Derivative):
         self._integrand = integrand
         self._coefficients = coefficients
         self._arguments = arguments
-        if isinstance(coefficient_derivatives, Data):
+        if isinstance(coefficient_derivatives, ExprMapping):
             self._coefficient_derivatives = coefficient_derivatives
         else:
-            self._coefficient_derivatives = Data(coefficient_derivatives)
+            self._coefficient_derivatives = ExprMapping(coefficient_derivatives)
 
     def operands(self):
         return (self._integrand, self._coefficients, self._arguments, self._coefficient_derivatives)
@@ -168,8 +165,8 @@ class Grad(CompoundDerivative):
 
     def __new__(cls, f):
         # Return zero if expression is trivially constant
-        dim = f.geometric_dimension()
         if f.is_cellwise_constant():
+            dim = f.geometric_dimension()
             free_indices = f.free_indices()
             index_dimensions = subdict(f.index_dimensions(), free_indices)
             return Zero(f.shape() + (dim,), free_indices, index_dimensions)
@@ -218,6 +215,64 @@ class Grad(CompoundDerivative):
     def __repr__(self):
         return "Grad(%r)" % self._f
 
+class ReferenceGrad(CompoundDerivative): # FIXME: Check implementation!
+    __slots__ = ("_f", "_dim",)
+
+    def __new__(cls, f):
+        # Return zero if expression is trivially constant
+        if f.is_cellwise_constant():
+            domain = f.domain()
+            dim = domain.topological_dimension()
+            free_indices = f.free_indices()
+            index_dimensions = subdict(f.index_dimensions(), free_indices)
+            return Zero(f.shape() + (dim,), free_indices, index_dimensions)
+        return CompoundDerivative.__new__(cls)
+
+    def __init__(self, f):
+        CompoundDerivative.__init__(self)
+        domain = f.domain()
+        dim = domain.topological_dimension()
+        self._f = f
+        self._dim = dim
+
+    def reconstruct(self, op):
+        "Return a new object of the same type with new operands."
+        if op.is_cellwise_constant():
+            ufl_assert(op.shape() == self._f.shape(),
+                       "Operand shape mismatch in ReferenceGrad reconstruct.")
+            ufl_assert(self._f.free_indices() == op.free_indices(),
+                       "Free index mismatch in ReferenceGrad reconstruct.")
+            return Zero(self.shape(), self.free_indices(),
+                        self.index_dimensions())
+        return self.__class__._uflclass(op)
+
+    def evaluate(self, x, mapping, component, index_values, derivatives=()):
+        "Get child from mapping and return the component asked for."
+        r = len(component)
+        component, i = component[:-1], component[-1]
+        derivatives = derivatives + (i,)
+        result = self._f.evaluate(x, mapping, component, index_values,
+                                  derivatives=derivatives)
+        return result
+
+    def operands(self):
+        return (self._f,)
+
+    def free_indices(self):
+        return self._f.free_indices()
+
+    def index_dimensions(self):
+        return self._f.index_dimensions()
+
+    def shape(self):
+        return self._f.shape() + (self._dim,)
+
+    def __str__(self):
+        return "reference_grad(%s)" % self._f
+
+    def __repr__(self):
+        return "ReferenceGrad(%r)" % self._f
+
 class Div(CompoundDerivative):
     __slots__ = ("_f",)
 
@@ -259,8 +314,8 @@ class NablaGrad(CompoundDerivative):
 
     def __new__(cls, f):
         # Return zero if expression is trivially constant
-        dim = f.geometric_dimension()
         if f.is_cellwise_constant():
+            dim = f.geometric_dimension()
             free_indices = f.free_indices()
             index_dimensions = subdict(f.index_dimensions(), free_indices)
             return Zero((dim,) + f.shape(), free_indices, index_dimensions)
