@@ -8,7 +8,7 @@ as well as the transpose "A.T" and spatial derivative "a.dx(i)".
 This is to avoid circular dependencies between Expr and its subclasses.
 """
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -26,9 +26,6 @@ This is to avoid circular dependencies between Expr and its subclasses.
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008
-#
-# First added:  2008-03-14
-# Last changed: 2012-03-20
 
 #--- The base object for all UFL expression tree nodes ---
 
@@ -44,7 +41,7 @@ def print_expr_statistics():
 class Expr(object):
     "Base class for all UFL objects."
     # Freeze member variables for objects of this class
-    __slots__ = ()
+    __slots__ = () # "_hash",)
 
     _class_usage_statistics = defaultdict(int)
     _class_del_statistics = defaultdict(int)
@@ -87,37 +84,38 @@ class Expr(object):
         "Return the tensor rank of the expression."
         return len(self.shape())
 
+    # All subclasses must implement domains if it is known
+    def domains(self):
+        # TODO: Is it better to use an external traversal algorithm for this?
+        from ufl.geometry import extract_domains
+        return extract_domains(self)
+
     # All subclasses must implement domain if it is known
-    def domain(self): # TODO: Is it better to use an external traversal algorithm for this?
-        "Return the domain this expression is defined on."
-        result = None
-        for o in self.operands():
-            domain = o.domain().top_domain()
-            if domain is not None:
-                result = domain # Best we have so far
-                cell = domain.cell()
-                if cell is not None:
-                    # A domain with a fully defined cell, we have a winner!
-                    break
-        return result
+    def domain(self):
+        "Return the single unique domain this expression is defined on or throw an error."
+        domains = self.domains()
+        if len(domains) == 1:
+            domain, = domains
+            return domain
+        elif domains:
+            error("Found multiple domains, cannot return just one.")
+        else:
+            return None
 
     # All subclasses must implement cell if it is known
     def cell(self): # TODO: Deprecate this
         "Return the cell this expression is defined on."
-        for o in self.operands():
-            cell = o.cell()
-            if cell is not None:
-                return cell
-        return None
+        domain = self.domain()
+        return domain.cell() if domain is not None else None
 
     # This function was introduced to clarify and
     # eventually reduce direct dependencies on cells.
-    def geometric_dimension(self): # TODO: Deprecate this, use external analysis algorithm
+    def geometric_dimension(self):
         "Return the geometric dimension this expression lives in."
-        cell = self.cell()
-        if cell is None:
-            error("Cannot get geometric dimension from an expression with no cell!")
-        return cell.geometric_dimension()
+        # TODO: Deprecate this, and use external analysis algorithm?
+        for domain in self.domains():
+            return domain.geometric_dimension()
+        error("Cannot get geometric dimension from an expression with no domains!")
 
     def is_cellwise_constant(self):
         "Return whether this expression is spatially constant over each cell."
@@ -125,9 +123,9 @@ class Expr(object):
 
     #--- Functions for float evaluation ---
 
-    def evaluate(self, x, mapping, component, index_values, derivatives=()):
+    def evaluate(self, x, mapping, component, index_values):
         """Evaluate expression at given coordinate with given values for terminals."""
-        raise NotImplementedError(self.__class__.evaluate)
+        error("Symbolic evaluation of %s not available." % self._uflclass.__name__)
 
     def __float__(self):
         if self.shape() != () or self.free_indices() != ():
@@ -151,7 +149,7 @@ class Expr(object):
 
     # All subclasses must implement signature_data
     def signature_data(self):
-        "Return data that uniquely identifies this object."
+        "Return data that uniquely identifies form compiler relevant aspects of this object."
         raise NotImplementedError(self.__class__.signature_data)
 
     # All subclasses must implement __repr__
