@@ -1,6 +1,6 @@
 "This module defines the UFL finite element classes."
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -19,9 +19,11 @@
 #
 # Modified by Kristian B. Oelgaard
 # Modified by Marie E. Rognes 2010, 2012
+# Modified by Andrew T. T. McRae 2014
+# Modified by Lawrence Mitchell 2014
 #
 # First added:  2008-03-03
-# Last changed: 2012-08-16
+# Last changed: 2014-03-18
 
 from ufl.finiteelement.finiteelementbase import FiniteElementBase
 from ufl.finiteelement.finiteelement import FiniteElement
@@ -31,3 +33,6 @@ from ufl.finiteelement.mixedelement import TensorElement
 from ufl.finiteelement.enrichedelement import EnrichedElement
 from ufl.finiteelement.restrictedelement import RestrictedElement
 from ufl.finiteelement.tensorproductelement import TensorProductElement
+from ufl.finiteelement.outerproductelement import OuterProductElement
+from ufl.finiteelement.outerproductelement import OuterProductVectorElement
+from ufl.finiteelement.hdivcurl import HDiv, HCurl
