@@ -1,7 +1,7 @@
 """This module provides functions used by the UFL implementation to
 output messages. These may be redirected by the user of UFL."""
 
-# Copyright (C) 2005-2013 Anders Logg and Martin Sandve Alnaes
+# Copyright (C) 2005-2014 Anders Logg and Martin Sandve Alnaes
 #
 # This file is part of UFL.
 #
@@ -19,9 +19,6 @@ output messages. These may be redirected by the user of UFL."""
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Johan Hake, 2009.
-#
-# First added:  2005-02-04
-# Last changed: 2011-06-08
 
 import sys
 import types
@@ -233,3 +230,5 @@ ufl_logger = Logger("UFL", UFLException)
 
 for foo in log_functions:
     exec("%s = ufl_logger.%s" % (foo, foo))
+
+set_level(DEPRECATE)
