@@ -3,7 +3,7 @@ which are either already available as member functions on UFL objects
 or defined as compound operators involving basic operations on the UFL
 objects."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes and Anders Logg
+# Copyright (C) 2008-2014 Martin Sandve Alnes and Anders Logg
 #
 # This file is part of UFL.
 #
@@ -21,9 +21,6 @@ objects."""
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Kristian B. Oelgaard, 2011
-#
-# First added:  2008-04-09
-# Last changed: 2013-03-15
 
 import operator
 from ufl.log import error, warning
@@ -276,10 +273,10 @@ def Dn(f):
     """UFL operator: Take the directional derivative of f in the
     facet normal direction, Dn(f) := dot(grad(f), n)."""
     f = as_ufl(f)
-    cell = f.cell()
-    if cell is None: # FIXME: Rather if f.is_cellwise_constant()?
+    if f.is_cellwise_constant():
         return Zero(f.shape(), f.free_indices(), f.index_dimensions())
-    return dot(grad(f), cell.n)
+    from ufl.geometry import FacetNormal
+    return dot(grad(f), FacetNormal(f.domain()))
 
 def diff(f, v):
     """UFL operator: Take the derivative of f with respect to the variable v.
@@ -308,7 +305,7 @@ def grad(f):
 
     This operator follows the grad convention where
 
-      grad(s)[i] = s.dx(j)
+      grad(s)[i] = s.dx(i)
 
       grad(v)[i,j] = v[i].dx(j)
 
@@ -316,6 +313,8 @@ def grad(f):
 
     for scalar expressions s, vector expressions v,
     and arbitrary rank tensor expressions T.
+
+    See also: :py:func:`nabla_grad`
     """
     f = as_ufl(f)
     return Grad(f)
@@ -331,6 +330,8 @@ def div(f):
 
     for vector expressions v, and
     arbitrary rank tensor expressions T.
+
+    See also: :py:func:`nabla_div`
     """
     f = as_ufl(f)
     return Div(f)
@@ -340,7 +341,7 @@ def nabla_grad(f):
 
     This operator follows the grad convention where
 
-      nabla_grad(s)[i] = s.dx(j)
+      nabla_grad(s)[i] = s.dx(i)
 
       nabla_grad(v)[i,j] = v[j].dx(i)
 
@@ -348,6 +349,8 @@ def nabla_grad(f):
 
     for scalar expressions s, vector expressions v,
     and arbitrary rank tensor expressions T.
+
+    See also: :py:func:`grad`
     """
     f = as_ufl(f)
     return NablaGrad(f)
@@ -363,6 +366,8 @@ def nabla_div(f):
 
     for vector expressions v, and
     arbitrary rank tensor expressions T.
+
+    See also: :py:func:`div`
     """
     f = as_ufl(f)
     return NablaDiv(f)
@@ -378,15 +383,8 @@ rot = curl
 def jump(v, n=None):
     "UFL operator: Take the jump of v across a facet."
     v = as_ufl(v)
-    cell = v.cell()
-    if cell is None:
-        warning("TODO: Not all expressions have a cell. Is it right to return zero from jump then?")
-        # TODO: Is this right? If v has no cell, it doesn't depend on
-        # anything spatially variable or any form arguments, and thus
-        # the jump is zero. In other words, I'm assuming that
-        # "v.cell() is None" is equivalent with "v is a constant".
-        return Zero(v.shape(), v.free_indices(), v.index_dimensions())
-    else:
+    is_not_constant = len(v.domains()) > 0 # FIXME: Not quite right...
+    if is_not_constant:
         if n is None:
             return v('+') - v('-')
         r = v.rank()
@@ -394,6 +392,14 @@ def jump(v, n=None):
             return v('+')*n('+') + v('-')*n('-')
         else:
             return dot(v('+'), n('+')) + dot(v('-'), n('-'))
+    else:
+        warning("Returning zero from jump of expression without a domain. This may be erroneous.")
+        # FIXME: Is this right? If v has no cell, it doesn't depend on
+        # anything spatially variable or any form arguments, and thus
+        # the jump is zero. In other words, I'm assuming that
+        # "not v.domains()" is equivalent with "v is a constant".
+        # Update: This is NOT true for jump(Expression("x[0]")) from dolfin.
+        return Zero(v.shape(), v.free_indices(), v.index_dimensions())
 
 def avg(v):
     "UFL operator: Take the average of v across a facet."
@@ -586,16 +592,19 @@ def exterior_derivative(f):
 
     # Extract the element from the input f
     if isinstance(f, Indexed):
-        if len(f._indices) > 1:
+        expression, indices = f.operands()
+        if len(indices) > 1:
             raise NotImplementedError
-        index = int(f._indices[0])
-        element = f._expression.element()
+        index = int(indices[0])
+        element = expression.element()
         element = element.extract_component(index)[1]
     elif isinstance(f, ListTensor):
-        if len(f._expressions[0]._indices) > 1:
+        f0 = f.operands()[0]
+        f0expr, f0indices = f0.operands() # FIXME: Assumption on type of f0!!!
+        if len(f0indices) > 1:
             raise NotImplementedError
-        index = int(f._expressions[0]._indices[0])
-        element = f._expressions[0]._expression.element()
+        index = int(f0indices[0])
+        element = f0expr.element()
         element = element.extract_component(index)[1]
     else:
         try:
@@ -605,7 +614,7 @@ def exterior_derivative(f):
 
     # Extract the family and the
     family = element.family()
-    gdim = element.cell().geometric_dimension()
+    gdim = element.domain().geometric_dimension()
 
     # L^2 elements:
     if "Disc" in family:
