@@ -1,7 +1,7 @@
 """This module defines the Terminal class, the superclass
 for all types that are terminal nodes in the expression trees."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -33,10 +33,11 @@ from ufl.common import counted_init
 
 class Terminal(Expr):
     "A terminal node in the UFL expression tree."
-    __slots__ = ()
+    __slots__ = ("_hash",)
 
     def __init__(self):
         Expr.__init__(self)
+        self._hash = None
 
     def reconstruct(self, *operands):
         "Return self."
@@ -55,6 +56,10 @@ class Terminal(Expr):
         "A Terminal object never has free indices."
         return EmptyDict
 
+    def domains(self):
+        "Return tuple of domains related to this terminal object."
+        raise NotImplementedError("Missing implementation of domains().")
+
     def evaluate(self, x, mapping, component, index_values, derivatives=()):
         "Get self from mapping and return the component asked for."
         f = mapping.get(self)
@@ -93,10 +98,14 @@ class Terminal(Expr):
         return f
 
     def signature_data(self):
+        "Default signature data for of terminals just return the repr string."
         return repr(self)
 
     def __hash__(self):
-        return hash(repr(self))
+        "Default hash of terminals just hash the repr string."
+        if self._hash is None:
+            self._hash = hash(repr(self))
+        return self._hash
 
     def __eq__(self, other):
         "Default comparison of terminals just compare repr strings."
@@ -110,12 +119,8 @@ class Terminal(Expr):
 
 class FormArgument(Terminal):
     __slots__ = ()
-    def __init__(self, count=None, countedclass=None):
+    def __init__(self):
         Terminal.__init__(self)
-        counted_init(self, count, countedclass)
-
-    def count(self):
-        return self._count
 
 class UtilityType(Terminal):
     __slots__ = ()
@@ -134,18 +139,6 @@ class UtilityType(Terminal):
     def is_cellwise_constant(self):
         error("Calling is_cellwise_constant on a utility type is an error.")
 
-#--- Non-tensor types ---
-
-class Data(UtilityType):
-    "For internal use, never to be created by users."
-    __slots__ = ("_data",)
-    def __init__(self, data):
-        UtilityType.__init__(self)
-        self._data = data
-
-    def __str__(self):
-        return "Data(%s)" % str(self._data)
-
-    def __repr__(self):
-        return "Data(%r)" % (self._data,)
-
+    def domains(self):
+        "Return tuple of domains related to this terminal object."
+        return ()
