@@ -1,7 +1,7 @@
 """Defines the Variable and Label classes, used to label
 expressions as variables for differentiation."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -81,11 +81,8 @@ class Variable(WrapperType):
     def shape(self):
         return self._expression.shape()
 
-    def cell(self):
-        return self._expression.cell()
-
-    def domain(self):
-        return self._expression.domain()
+    def domains(self):
+        return self._expression.domains()
 
     def is_cellwise_constant(self):
         return self._expression.is_cellwise_constant()
